package be.iminds.ilabt.jfed.experiment.setup.config;

import com.fasterxml.jackson.annotation.*;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.Objects;

@JsonTypeInfo(
        use = JsonTypeInfo.Id.NAME,
        include = JsonTypeInfo.As.PROPERTY,
        property = "source",
        defaultImpl = ProvidedContentRequestRSpec.class,
        visible = true)
@JsonSubTypes({
        @JsonSubTypes.Type(value = ProvidedUrlRequestRSpec.class, name = "PROVIDE_URL"),
        @JsonSubTypes.Type(value = ProvidedFileRequestRSpec.class, name = "PROVIDE_FILE"),
        @JsonSubTypes.Type(value = ProvidedContentRequestRSpec.class, name = "PROVIDE_CONTENT")
})
@JsonInclude(JsonInclude.Include.NON_DEFAULT)
@JsonIgnoreProperties(value={"@context"})
public abstract class ProvidedRequestRSpec extends RequestRSpec {
    @Nullable
    private final String providedContentSource; //url, file or content as specified in source

    private final boolean bindTestedServerToUnboundNodes;
    @Nullable private final String bindUnboundNodesUrn;

    public ProvidedRequestRSpec() {
        super(RequestRSpecSource.PROVIDE_CONTENT,
                false,
                false);
        this.providedContentSource = "";
        //best that bindTestedServerToUnboundNodes is default. For geni tutorial tests, this is very convenient.
        //Also, jFed Gui Logic doesn't work correctly for unbound nodes this anyway.
        this.bindTestedServerToUnboundNodes = true;
        this.bindUnboundNodesUrn = null;
    }

    @JsonCreator
    public ProvidedRequestRSpec(@Nonnull @JsonProperty("source") RequestRSpecSource source,
                                @Nullable @JsonProperty("providedContentSource") String providedContentSource,
                                @Nullable @JsonProperty("bindTestedServerToUnboundNodes") Boolean bindTestedServerToUnboundNodes,
                                @Nullable @JsonProperty("bindUnboundNodesUrn") String bindUnboundNodesUrn,
                                @Nullable @JsonProperty("autoAssignSpecificNodesUsingListResources") Boolean autoAssignSpecificNodesUsingListResources,
                                @Nullable @JsonProperty("useNitosLease") Boolean useNitosLease) {
        super(source, autoAssignSpecificNodesUsingListResources, useNitosLease);
        assert source == RequestRSpecSource.PROVIDE_CONTENT ||
                source == RequestRSpecSource.PROVIDE_FILE ||
                source == RequestRSpecSource.PROVIDE_URL;
        assert this.getSource() == RequestRSpecSource.PROVIDE_CONTENT ||
                this.getSource() == RequestRSpecSource.PROVIDE_FILE ||
                this.getSource() == RequestRSpecSource.PROVIDE_URL;
        this.providedContentSource = providedContentSource;
        this.bindTestedServerToUnboundNodes = bindTestedServerToUnboundNodes == null ?
                new ProvidedContentRequestRSpec().getBindTestedServerToUnboundNodes() :
                bindTestedServerToUnboundNodes;
        this.bindUnboundNodesUrn = bindUnboundNodesUrn;
    }

    @JsonIgnore
    public boolean isValid() {
        if (providedContentSource == null)
            throw new IllegalArgumentException("Config Error: rspec.providedContentSource may not be null if rspec.source == "+source);
        if (providedContentSource.trim().isEmpty())
            throw new IllegalArgumentException("Config Error: rspec.providedContentSource may not be empty if rspec.source == "+source);
        return true;
    }

    @JsonProperty
    @Nullable
    public String getProvidedContentSource() {
        return providedContentSource;
    }

    /**
     * This option makes only sense for automated tests.
     *
     * If this is true, it implies "bindUnboundNodesUrn" is "auto".
     * (so bindUnboundNodesUrn will be ignored)
     *
     * @return
     */
    @JsonProperty
    public boolean getBindTestedServerToUnboundNodes() {
        return bindTestedServerToUnboundNodes;
    }

    /**
     * Urn of the AM to bind unbound nodes to.
     * If null, unbound nodes should not be bound.
     * @return
     */
    @Nullable
    @JsonProperty
    public String getBindUnboundNodesUrn() {
        return bindUnboundNodesUrn;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof ProvidedRequestRSpec)) return false;
        if (!super.equals(o)) return false;
        ProvidedRequestRSpec that = (ProvidedRequestRSpec) o;
        return bindTestedServerToUnboundNodes == that.bindTestedServerToUnboundNodes &&
                Objects.equals(providedContentSource, that.providedContentSource) &&
                Objects.equals(bindUnboundNodesUrn, that.bindUnboundNodesUrn);
    }

    @Override
    public int hashCode() {
        return Objects.hash(super.hashCode(), providedContentSource, bindTestedServerToUnboundNodes, bindUnboundNodesUrn);
    }

    @Override
    public String toString() {
        return "ProvidedRequestRSpec{" +
                "source='" + getSource() + '\'' +
                ", providedContentSource=" + providedContentSource +
                ", bindTestedServerToUnboundNodes=" + bindTestedServerToUnboundNodes +
                ", bindUnboundNodesUrn=" + bindUnboundNodesUrn +
                '}';
    }
}
